/*  marshal.h -- simple esci marshalling protocol over tcp/ip
 *  Copyright (C) 2008, 2009  SEIKO EPSON CORPORATION
 *
 *  License: AVASYS Public License
 *  Authors: AVASYS CORPORATION
 *
 *  This file is part of the SANE backend distributed with Image Scan!
 *  It is distributed under the terms of the AVASYS PUBLIC LICENSE.
 *
 *  You should have received a verbatim copy of the AVASYS PUBLIC
 *  LICENSE along with the software.
 */


#ifndef included_marshal_h
#define included_marshal_h

#include <stdint.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*! defines for mapping exceptional return values from read()/write(); in
 *  particular, a return value of zero from read() conflicts with our
 *  marshal_read() API where a return value of zero does not indicate EOF,
 *  but a successful read of zero bytes.
 */
#define MERR -1 /*! read/write error */
#define MEOF -2 /*! eof; only returned by marshal_read() */

enum {
  TYPE_ESC = 1,
  TYPE_INIT,
  TYPE_DIE,
  TYPE_OPEN,
  TYPE_CLOSE,
  TYPE_LIST,
  TYPE_STATUS,
};

enum {
  STATUS_OK = 0,
  STATUS_NG
};

ssize_t marshal_write (int sock, uint16_t id, uint8_t type_status,
                       size_t size, const void* payload);
ssize_t marshal_read (int sock, uint16_t *id, uint8_t *type_status,
                      void** payload);


#ifdef __cplusplus
}	/* extern "C" */
#endif

#endif	/* !defined (included_marshal_h) */
