/* primax_scan -- linux driver for Primax scanners

$Id: libprimax.c,v 1.7 2002/11/07 12:43:43 onjo Exp $

Authors:
Marco Foglia <Marco.Foglia@switzerland.org>
   Thomas Schano <schano@t-online.de>
   Christian Ordig <chr.ordig@gmx.net>

   Copyright (C) 1999 Marco Foglia, Thomas Schano, Christian Ordig

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/        
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <unistd.h>            
#include <sched.h>
#include <sys/time.h>
#include <tiffio.h>
#include <getopt.h>
#include <errno.h>

#include "io.h"                        
#include "primax_scan.h"
#include "lp.h"
#include "tables.h"
#include "LM9811.h"
#include <time.h>
#include "libprimax.h"


/*void scan(scan_image *image, int mode);
  void init_image(scan_image **image);
  int save_calibration(scan_image *image);
  int load_calibration(scan_image *image);
  int read_calibration_data(scan_image *image); 
*/
/* void tiff_open(TIFF **tiffile, scan_image *image); */
/* void tiff_close(TIFF *tiffile); */
void wakeup(void);
int power_sequence(int sequence_code);
void wait_for_carriage(void);
int move_carrige_forward(float distance);
int move_carrige_backward(float distance);
void parse_options(scan_image *image, int argc, char **argv);


/*-------------------------------------------------------------*/
/*
 * switches the scanner online
 */
/* void wakeup_old(void) */
/* { */
/*     int i; */
/*     unsigned char data[] = {0x73, 0x72, 0x73, 0x73,  */
/* 			    0x72, 0x73, 0x73, 0x72}; */
    
/*     clear_timeout(); */

/*     io_outb(0x04, port + 2);   */
/*     io_outb(0x70, port); */
/*     io_outb(0x72, port); */
/*     for (i = 0; i < 8; i++){ */
/* 	io_outb(0x73, port); */
/* 	io_outb(0xF3, port);  */
/* 	io_outb(0x73, port); */
/* 	io_outb(0xF3, port); */
/* 	io_outb(0x73, port); */
/*     } */
/*     io_outb(0x7F, port); */
/*     io_outb(0x04, port + 2); */

/*     clear_timeout(); */

/*     io_outb(0x04, port + 2);   */
/*     io_outb(0x70, port); */
/*     io_outb(0x72, port); */
/*     for (i = 0; i < 8; i++){ */
/* 	io_outb(data[i], port); */
/* 	io_outb(data[i] + 0x80, port); */
/* 	io_outb(data[i], port); */
/* 	io_outb(data[i] + 0x80, port); */
/* 	io_outb(data[i], port); */
/*     } */
/*     io_outb(0x7F, port); */
/*     io_outb(0x04, port + 2); */

/*     DBG(DBG_LOW, "wakeup: done\n");     */
/* } */




/*-------------------------------------------------------------*/
/*  
 * This is the sequence begining with 10-18-10-.... 
 * not yet understood - but without it nothing happens
 *
 * power_sequence is DrvMcuSendRequest from the Win driver
 */


int power_sequence(int sequence_code)
{
    int i, status, timeout = 0;
    unsigned char back;
    long long timer;

    DBG(DBG_HIGH, "power_sequence:  code: %04x\n", sequence_code); 

    epp_read_one(&back, 0x0F);
    epp_write_one(0x10, 0x07); 
    epp_read_one(&back, 0x0F);
    epp_write_one(0x18, 0x07);
    epp_read_one(&back, 0x0F);
    
    for (i = 15; i >= 0 ; i--) {
	epp_write_one(0x10, 0x07);
	
	if (i!=15) {
	    init_timer(&timer, LOOP_TIMEOUT);
	    do 
		epp_read_one(&back, 0x0E); /* win driver drops line */
	    while ((back & 0x80) && !(timeout = check_timer(&timer)));  
	}	
	epp_read_one(&back, 0x0F); 
	epp_write_one(0x12 + ((sequence_code >> i) & 0x01), 0x07);
	epp_read_one(&back, 0x0F);
	
	init_timer(&timer, LOOP_TIMEOUT);
	do 
	    epp_read_one(&back, 0x0E);
	while ((back & 0x80) && !(timeout = check_timer(&timer)));  
	status = epp_read_one(&back, 0x0F);
    }
    epp_write_one(0x10, 0x07);

    if ((status == TIMEOUT) || (timeout == TIMEOUT))
	DBG(DBG_TIMEOUT | DBG_HIGH, 
	    "power_sequence: failed - TIMEOUT\n"); 

    return status;
}
void wait_for_carriage(void)
{
    unsigned char back;
    long long timer;
    
    init_timer(&timer, 15000000);
    do {
        scan_delay(75000);
        epp_read_one(&back, 0x0E);
    } while (!(back & 0x40) && !(check_timer(&timer)));
}



/*-------------------------------------------------------------*/

    
int prepare_scan(struct scan_session *sc, scan_image *image) {
    
    int i;

    sc->image = image;
    sc->oflag = BUFFSIZE-1;
    sc->pointer = 0;
    sc->red_ptr = 0;
    sc->green_ptr = 0;
    sc->blue_ptr = 0;
    sc->read_ptr = 0;
    sc->phys_line = 0;
    sc->width  = image->width;
    sc->ypixcount = image->height;
    sc->num_colors = image->num_colors;
    sc->resolution = image->resolution;
    sc->data = (char*)malloc ( sc->width * sizeof(char));
    if (sc->data == NULL) 
	return -1;

    for( i = 0; i < BUFFSIZE; i++)
        sc->line_status[i] = 0;
    
    if ( image->colormode & 0x0010 ) /* gray mode */
    {
        sc->total_width = image->width; /* only one color channel */
        sc->num_colors = 1;
        sc->offset_flag = 0;
    }
    else
    {
        sc->total_width = 3 * image->width; /* three color channels */
        sc->num_colors = 3;
        sc->offset_flag = 1;
    }
    
    /* allocate a ring buffer */
    sc->tiffdata = (unsigned char *)malloc( (sc->total_width) * BUFFSIZE ); 
    if (sc->tiffdata == NULL)
    
    /* find the start line */
    while(image->y_table_int[sc->phys_line] == 0x00 ) 
	sc->phys_line += 1;

    sc->y = 0;
    return 0;
}

unsigned char * get_scan_data(struct scan_session *sc, int *size) {
    
    int color; /* 1 = red ; 2 = green ; 4 = blue */
    unsigned char pa;
    unsigned char pb;
    unsigned char back;
    int i,j;
    unsigned char *retval;

    if (sc->y >= sc->ypixcount) { 
	/* scan finished */
	*size = -1;
	return NULL;
    }
    
    while( sc->line_status[sc->read_ptr] != ( 0x07 & sc->image->colormode )) 
        {
	    for ( color = 1; color < 8; color = color << 1){ /* for color = (1,2,4) */
	
		do { /* wait until fifo is filled */
		    back = epp_read_one(&pa, 0x06);  
		    back = epp_read_one(&pb, 0x05);
		} while ((( i = pa * 0x100 + pb) < (sc->total_width) -1 ));
		
                switch ( color & sc->image->y_table_int[ sc->phys_line ] ){
		case 1:
		    sc->pointer = sc->red_ptr * sc->total_width;
		    sc->line_status[sc->red_ptr] |= 1;
		    sc->red_ptr = ( sc->red_ptr + 1 ) & ( sc->oflag );
		    break;
		case 2:
		    sc->pointer = sc->green_ptr * sc->total_width + 1 * sc->offset_flag;
		    sc->line_status[ sc->green_ptr ] |= 2;
		    sc->green_ptr = ( sc->green_ptr + 1 ) &  ( sc->oflag );
		    break;
		case 4:
		    sc->pointer = sc->blue_ptr * sc->total_width + 2 * sc->offset_flag;
		    sc->line_status[ sc->blue_ptr ] |= 4;
		    sc->blue_ptr = ( sc->blue_ptr + 1 ) & ( sc->oflag );
		    break;
		}
		
                
                if (0xf & sc->image->y_table_int[sc->phys_line]){
                    epp_read( sc->data, sc->width, FIFO);
                    if ( color & sc->image->y_table_int[ sc->phys_line ] )
                        for (i = 0; i < sc->width ; i++){
                            sc->tiffdata[sc->pointer] = sc->data[i];
                            sc->pointer += sc->num_colors;
                        }
                }
            }
	    sc->phys_line += 1;
        }
    if ( sc->image->colormode & 0x0100 ) /* gray mode */
	{
	    sc->pix_read_count = sc->pix_write_count = sc->read_ptr * sc->total_width;
	    for (i = 0; i < sc->width ; i++)
		{
		    sc->akku = ( sc->tiffdata[ sc->pix_read_count    ] & 0xff ) * 30 +
			( sc->tiffdata[ sc->pix_read_count + 1 ] & 0xff) * 59 +
			( sc->tiffdata[ sc->pix_read_count + 2 ] & 0xff) * 11 ;
		    sc->tiffdata[ sc->pix_read_count     ] = 0x00;
		    sc->tiffdata[ sc->pix_read_count + 1 ] = 0x00;
		    sc->tiffdata[ sc->pix_read_count + 2 ] = 0x00;
		    sc->tiffdata[ sc->pix_write_count ] = ( sc->akku / 100 ) & 0xff;
		    sc->pix_read_count += sc->num_colors;
		    sc->pix_write_count += 1;
		}
	}
    
    if ( sc->image->colormode & 0x1000 ) /* one bit mode */
	{
	    sc->pix_read_count = sc->pix_write_count = sc->read_ptr * sc->total_width;
	    for (i = 0; i < (sc->width / 8) ; i++)
		{
		    sc->akku= 0;
		    for (j = 0; j < 8 ; j++)
			{
			    sc->akku <<= 1;
			    if (( sc->tiffdata[ sc->pix_read_count ] & 0xff) > 0x7f)
				sc->akku |= 0x01;
			    sc->pix_read_count += 1;
			}
		    sc->tiffdata[ sc->pix_write_count ] = ( sc->akku )  & 0xff;
		    sc->pix_write_count += 1;
		}
	}
    
    *size = sc->total_width;
    retval = sc->tiffdata + sc->read_ptr * sc->total_width;    
    
    sc->line_status[ sc->read_ptr ] = 0;
    sc->read_ptr = ( sc->read_ptr + 1 ) & sc->oflag;
    sc->y += 1;
    return retval;
}

void finish_scan(struct scan_session *sc) {
    free( sc->data );
    free( sc->tiffdata );
}


int read_calibration_data(scan_image *image)
{
    unsigned char pa, pb;
    int back;
    unsigned int pointer = 0, y, i;
    unsigned int height = image->height;
    unsigned int total_width = image->num_colors * image->width;

    for (y = 0; y < height ; y++){
        do {
	    if ( ( back = epp_read_one(&pa, 0x06) ) != 0 ) {
		printf("error\n");	
		return -1;
	    }
	    if ( ( back = epp_read_one(&pb, 0x05) ) != 0 ) {
		printf("error\n");
		return -1;
	    }
        } while ( ( ( i = pa * 0x100 + pb) < total_width -1 ) );
        back = epp_read( (image->data) + pointer, total_width, FIFO );
	if ( back != 0 ) {
	    printf("error\n");
	    return -1;
	}
        pointer += total_width;
    }
    return 0;
}
/*-------------------------------------------------------------*/
/*
 * moves the lamp forward. distance in inch
 *
 * same as DrvMoveCarrigeForward  
 * theory: 
 *    - power 0x2000
 *    - out 5    - 0x00, 0x0D, 0x0D, 0x0D
 *    - in  0x0F
 *    - out 5    - 0x00, 0x0D, 0x0D, 0x0D
 *    - out 6    - 0x24, 0x20, 0x24, 0xBC
 *    - power 0x2000
 *    - out 5    - 0x00, 0x0D, 0x0D, 0x0D
 *    - out 5    - 0x00, 0x00, 0x00, 0x00
 *    - out 6    - 0x24, 0x20, 0x24, 0x3C
 *    - out 5    - 0x00, 0x00, 0x00, 0x00
 *    - out 6    - 0x24, 0x20, 0x24, 0x3C
 *    - power 0x2000
 *    - out 5    - 0x00, 0x00, 0x00, 0x00
 *    - out 5    - 0x0D, 0x0D, 0x0D, 0x0D
 *    - out 6    - 0x24, 0x20, 0x24, 0x3C
 *    - out 6    - 0x24, 0x20, 0x24, 0x3C
 *    - out 6    - 0x26, 
 *    - out 6    - 0x24, 
 *    [- in  0x0E          
 *     - in  0x0E    - repeat !0x80
 *     - out 6   - 0x24
 *     - out 6   - 0x26
 *     -]out 6   - 0x24  - reapeat [..]
 *    -  out 5   - 0x0D, 0x0D, 0x0D, 0x0D
 *    - power 0x40B0
 *    - power 0x8000
 *
 * values are different for 150 dpi !!!
 */
int move_carrige_forward(float distance) 
{
    int i,status, timeout = 0;
    long long timer;
    unsigned char back;
#if USE_SCHEDULER
    struct sched_param priority;
#endif

    /* change this if you know the meaning*/
    unsigned char value_1 = 0x00, value_2 = 0x24;

    power_sequence(S_2000);

    epp_write_one(value_1, 0x05);
    status =  epp_read_one(&back, 0x05);
    epp_write_one(value_1, 0x05);
    epp_write_one(value_2, 0x06);

    power_sequence(S_2000);

    epp_write_one(value_1, 0x05);
    epp_write_one(0x00, 0x05);
    epp_write_one(value_2 & 0x7F, 0x06);
    epp_write_one(0x00, 0x00);
    epp_write_one(value_2 & 0x7F, 0x06);
    
    power_sequence(S_2000);

    epp_write_one(0x00, 0x05);
    epp_write_one(0x05, 0x05);                 /* Speed ! */ 
    epp_write_one(value_1, 0x06);
    epp_write_one(value_1, 0x06);
    epp_write_one(0x26, 0x06);    
    epp_write_one(0x24, 0x06);

    /* we need this to make a smooth move */
#if USE_SCHEDULER
    priority.sched_priority = 1;
    sched_setscheduler(0, SCHED_FIFO, &priority);
#endif

    for (i = 0; i < distance * Y_RESOLUTION; i++) {
	status = epp_read_one(&back, 0x0E);
	init_timer(&timer, LOOP_TIMEOUT);
	do {
	    status = epp_read_one(&back, 0x0E);  
	} while ((back & 0x80) && !(timeout = check_timer(&timer))); /* ready for next move? */	
	if (timeout)
	    DBG(DBG_TIMEOUT | DBG_MEDIUM, "move_carrige_forward:  timeout!\n");
	epp_write_one(0x24, 0x06);
	epp_write_one(0x26, 0x06);    /* move */ /*bit 15 is direction bit */
	epp_write_one(0x24, 0x06);
    }
    epp_write_one(0x0D, 0x05);
#if USE_SCHEDULER
    priority.sched_priority = 0;
    sched_setscheduler(0, SCHED_OTHER, &priority);
#endif
    power_sequence(S_40B0);
    power_sequence(S_8000);
    return status;
}


/*-------------------------------------------------------------*/




/*-------------------------------------------------------------*/
/*
 * moves the lamp backward. distance in inch
 *
 * it is not a windows trace, it's just
 * move_carrige_forward with other values
 */

int move_carrige_backward(float distance) 
{  
    int i, status, timeout = NO_TIMEOUT;
    long long timer;
    unsigned char back;
#if USE_SCHEDULER
    struct sched_param priority;
#endif

    /* change this if you know the meaning*/
    unsigned char value_1 = 0x00, value_2 = 0x24;
    
    power_sequence(S_2000);
    
    epp_write_one(value_1, 0x05);
    status =  epp_read_one(&back, 0x05);
    epp_write_one(value_1, 0x05);
    epp_write_one(value_2, 0x06);
    
    power_sequence(S_2000);
    
    epp_write_one(value_1, 0x05);
    epp_write_one(0x00, 0x05);
    epp_write_one(value_2 & 0x7F, 0x06); /* 0x7F just for fun */
    epp_write_one(0x00, 0x00);
    epp_write_one(value_2 & 0x7F, 0x06);
    
    power_sequence(S_2000);
    
    epp_write_one(0x00, 0x05);
    epp_write_one(0x00, 0x05);                 /* Speed ? */ 
    epp_write_one(value_1, 0x06);
    epp_write_one(value_1, 0x06);
    epp_write_one(0x22, 0x06);    
    epp_write_one(0x20, 0x06);
    /* we need this to make a smooth move */
#if USE_SCHEDULER
    priority.sched_priority = 1;
    sched_setscheduler(0, SCHED_FIFO, &priority);
#endif
    for (i = 0; i < distance * Y_RESOLUTION; i++) {
	status = epp_read_one(&back, 0x0E);
	init_timer(&timer, LOOP_TIMEOUT);
	do {
	    status = epp_read_one(&back, 0x0E);  
	} while ((back & 0x80) && !(timeout = check_timer(&timer))); /* ready for next move? */
	if (timeout)
	    DBG(DBG_TIMEOUT | DBG_MEDIUM, "move_carrige_backward:  timeout!\n");
	epp_write_one(0x20, 0x06);
	epp_write_one(0x22, 0x06);    /* move */ /*bit 15 is direction bit */
	epp_write_one(0x20, 0x06);
    }
    epp_write_one(0x0D, 0x05);
#if USE_SCHEDULER
    priority.sched_priority = 0;
    sched_setscheduler(0, SCHED_OTHER, &priority);
#endif
    power_sequence(S_40B0);
    power_sequence(S_8000);
    return status;
}



/*-------------------------------------------------------------*/
/* Saves calibration data into image->save_calibration_file
 * Returns 1 if succesfull
 * Returns 0 and sets errno if not
 */
int save_calibration(scan_image *image){
    FILE *file;
    int pix_count;
    if ( ( file=fopen(image->save_calibration_file, "w") ) != NULL ){
	for (pix_count = 0; pix_count < LM9811_PGA_array_size; pix_count++)
	    fprintf (file, "%x ", image->LM9811_arr_PGA[pix_count]);
        fprintf (file, "\n");
    	fprintf (file, "%x\n", image->LM9811_reg_PGA);
	fprintf (file, "%x\n", image->LM9811_reg_VGA);
	fprintf (file, "%x\n", image->LM9811_reg_DAC);
	
	if (fclose(file)!=0)
	    return 0;
	return 1;
    }
    return 0;
}

/*-------------------------------------------------------------*/
/* Error proof reading of next hex from calib. file
 */
int read_next_calibration_value(FILE *file, unsigned char *val){
    unsigned int tmp_val;
    int result;
    result=fscanf (file, "%x", &tmp_val);
    if (result == 0  ||  result == EOF)
	return 0;
    *val = tmp_val;
    return 1;
}


/*-------------------------------------------------------------*/
/* Loads calibration data from image->load_calibration_file
 * Returns 1 if succesfull
 * Returns 0 and sets errno if not
 */
int load_calibration(scan_image *image){
    FILE *file;
    int pix_count;

    if ( ( file=fopen(image->load_calibration_file, "r") ) == NULL )
	return 0;
    for (pix_count = 0; pix_count < LM9811_PGA_array_size; pix_count++)
	if (!read_next_calibration_value (file, &(image->LM9811_arr_PGA[pix_count])))
	    return 0;
    if (!read_next_calibration_value (file, &(image->LM9811_reg_PGA))) 
	return 0;
    if (!read_next_calibration_value (file, &(image->LM9811_reg_VGA))) 
	return 0;
    if (!read_next_calibration_value (file, &(image->LM9811_reg_DAC))) 
	return 0;
    if (fclose(file)!=0)
	return 0;
    return 1;
    
}



/*-------------------------------------------------------------*/
/*
 *  ....
 */



void prepare_scanner( scan_image *image,int mode )
{
    int i;
//    unsigned char back;
    
    calc_y_table(image);
    calc_x_table(image);
    if ( !image->custom_gamma_table)
	calc_color_table(image);
    
    /* skip calibration area */
    if (mode == SCAN)
	move_carrige_forward( CALIBRATION_ARRAY_SIZE );
    else
        move_carrige_forward(image->from_top);
    
/*      move_carrige_forward(0); */
/*      move_carrige_forward(image->from_top); */
    
    write_LM9811( image->LM9811_reg_PGA , LM9811_PGA );
    write_LM9811( image->LM9811_reg_VGA , LM9811_VGA );
    write_LM9811( image->LM9811_reg_DAC , LM9811_DAC );

    
    epp_write(image->LM9811_arr_PGA, LM9811_PGA_array_size, CALIBRATION_TABLE);

         /*Reset FIFO*/
    epp_write_one(0x64, 0x06);
    epp_write_one(0x24, 0x06);  


    for(i = 0; i < 3; i++)
        epp_write((image->color_table)[i], color_table_size, COLOR_TABLE); 
    
    epp_write(image->x_table, x_table_size, X_TABLE); 
    epp_write(image->y_table, y_table_size, Y_TABLE);
    
    epp_write_one(0x40 | (image->speed & 0xf), 0x05);
    /* The bits 0..3 are for brightness,
       integration-time of ccd */
    
    //    epp_write_one(0xBC, 0x06); /* bit 5 = lamp on */
    epp_write_one(0xbD, 0x06); /* bit 0 = start scan, bit 7 = must be,
                        	  bit 6 = must not be, bit 5 = lamp on,
                                  bit 4 has to do something with calib. */

}


void finish_scanner(void){
    power_sequence(CARRIGE_RETURN);
    //    if (mode != SCAN) // don't wait until carriage is at home
    wait_for_carriage();
}

void scan(scan_image *image, int mode){
    prepare_scanner(image, mode);
  
    if (mode != SCAN)
	read_calibration_data(image);
  
    finish_scanner();
}

static int px_switch_lamp(int switch_lamp ) {

    unsigned char back, status;
    
    
    if ( epp_on() != EPP_ON_OK )
	return -1;
    wakeup();
    epp_write_one(switch_lamp, 0x06);	
    status = epp_read_one(&back, 0x0E);

    if (status != NO_TIMEOUT)
	return -1;
    
    power_sequence(GO_OFFLINE);
    epp_off();
    return back;
}

int lamp_on(void) {
    return px_switch_lamp( 0x20 );
}

int lamp_off(void) {
    return px_switch_lamp( 0x00 );
}


/*-------------------------------------------------------------*/
void init_image(scan_image **image) 
{
    int i;

    *image = malloc (sizeof(scan_image));

    (*image)->resolution = 50;
    (*image)->width = (*image)->resolution * MAX_WIDTH;
    (*image)->height = (*image)->resolution * MAX_HEIGHT ;
    (*image)->speed = -1;
    (*image)->from_top = 0;
    (*image)->from_left = 0;
    (*image)->num_colors = 3;
    (*image)->colormode = RGB_MODE;

    for (i = 0; i < 3; i++) {
	((*image)->color)[i].gamma = 1;
	((*image)->color)[i].contrast = 0;
	((*image)->color)[i].brightness = 0;
	((*image)->color_table)[i] = malloc(color_table_size);
    }
    (*image)->custom_gamma_table = 0;

    (*image)->LM9811_arr_PGA = calloc(1,LM9811_PGA_array_size);
    (*image)->LM9811_reg_PGA = 0;
    (*image)->LM9811_reg_VGA = 0;
    (*image)->LM9811_reg_DAC = 0;
    (*image)->y_table =(char*) malloc(y_table_size + 1 +BLUE_OFFSET );
    (*image)->y_table_int =(char*) malloc(y_table_size + 1 + BLUE_OFFSET);
    (*image)->x_table =(char*) malloc(x_table_size + 1 + BLUE_OFFSET);

    (*image)->data = NULL;

    /* (*image)->filename = malloc(100); */
    (*image)->filename = "primax_scan.tif";
    (*image)->lzw = 0xffff;

    (*image)->load_calibration_file =  CALIB_DATA_FILE;
    (*image)->save_calibration_file =  CALIB_DATA_FILE; 
}

void free_image(scan_image *image) {
    free(image->color_table[0]);
    free(image->color_table[1]);
    free(image->color_table[2]);
    free(image->LM9811_arr_PGA);
    free(image->y_table);
    free(image->y_table_int);
    free(image->x_table);
    free(image);
}


  
/* This stuff is for emacs
 * Local variables:
 * c-basic-offset: 4
 * End:       
 */
